﻿-- Priority List
-- by Klay EU-Hakkar
debug = false

PriorityList = LibStub("AceAddon-3.0"):NewAddon("PriorityList", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local addon	= LibStub("AceAddon-3.0"):GetAddon("PriorityList")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
LibStub("AceSerializer-3.0"):Embed(PriorityList)
LibStub("AceComm-3.0"):Embed(PriorityList)

local tinsert, tconcat, tremove = table.insert, table.concat, table.remove
local fmt, tostring, string_char = string.format, tostring, string.char
local select, pairs, next, type, unpack = select, pairs, next, type, unpack
local loadstring, assert, error = loadstring, assert, error
local setmetatable, getmetatable, rawset, rawget = setmetatable, getmetatable, rawset, rawget
local bit_band, bit_lshift, bit_rshift = bit.band, bit.lshift, bit.rshift

-- list version: this will be checked whenever the list has to be imported/exported
-- increment this number if the list structure changes
-- and apply a list upgrade fix in Init()
 PriorityList.version = 1000
--

local addonName, addonTable = ...
local L = addonTable.L

-- Constants --
-- sys message color
smc_r = 0.2
smc_g = 0.7
smc_b = 1
-- string colors
local LIGHT_BLUE  = "|cff00ccff"
local LIGHT_RED   = "|cffff6060"
local GREEN       = "|cff71C671"
local LIGHT_GREEN = "|cff71FF71"
local YELLOW      = "|cffffff00"
local GREY        = "|cff888888"
local GOLD        = "|cffffcc00"
local RED         = "|cffff0000"
local function AddColor(str,color)
 return color..str.."|r"
end

-- BOSS DROPS TABLE
 boss_drops_table = 
  { --NORMAL
   ["#1 Morchok"] = {
    ["Normal"] = {77213,77265,77263,77262,77214,77261,77212,77271,77266,77267,77268,77269,77270},
    ["Heroic"] = {78366,78365,78368,78372,78362,78370,78367,78373,78361,78364,78363,78371,78369}},
   ["#2 Warlord Zon'ozz"] = {
    ["Normal"] = {77215,77258,77257,77255,77259,77260,77204,77216},
    ["Heroic"] = {78391,78388,78393,78390,78389,77989,78387,78392}},
   ["#3 Yor'sahj, the Unsleeping"] = {
    ["Normal"] = {77206,77253,77252,77219,77203,77218,77254,77217},
    ["Heroic"] = {78402,78405,78406,78403,78401,78404,77991,77990}},
   ["#4 Hagara the Stormbinder"] = {
    ["Normal"] = {78012,78011,77221,77220,77251,77250,77249,77248},
    ["Heroic"] = {78417,78420,78415,78416,78419,78413,78418,78414}},
   ["#5 Ultraxion"] = {
    ["Normal"] = {77245,78013,77246,77247,77244,77243,77205,77223,77242},
    ["Heroic"] = {78434,78435,78430,78436,78431,78432,77992,78433,78429}},
   ["#6 Warmaster Blackhorn"] = {
    ["Normal"] = {77227,77226,77241,77225,77240,77224,77239,77202,77234},
    ["Heroic"] = {78449,78446,78447,78445,78451,78452,78448,78450,77993}},
   ["#7 Spine of Deathwing"] = {
    ["Normal"] = {78357,77197,77198,77199,77238,77237,77236,77235,77201,77200},
    ["Heroic"] = {78461,78462,78463,78464,78465,78997,78996,78995,78994,78998}},
   ["#8 Madness of Deatwing"] = {
    ["Normal"] = {78359,77196,77195,77194,77193,77191,77190,77189,77188},
    ["Heroic"] = {78474,78475,78479,78478,78472,78476,78473,78477,78471}},
   ["-Trash mobs"] = {
    ["Trash mobs"] = {78879,78884,78882,78886,78885,78887,78888,78889,77938,77192}},
   ["-Shared boss loot (#1 to #6)"] = {
    ["Normal"] = {77230,77228,77232,77229,77231,77208,77210,77207,77211,77209},
    ["Heroic"] = {78492,78493,78490,78489,78491,78002,78000,78003,77999,78001}}
   }
  
-- variables --
waiting_ids = {}
addon.optionsFrame = {}
addon.options = nil
opt_selected = ""
import_var = {
              sender = "",
              ready  = false,
              Items_List    = {},
              flagged_for_del = {},
              import_mode = false,
              counter = 0,
             }
-----------------



function PriorityList:SysMessage(msg)
 if debug then
  DEFAULT_CHAT_FRAME:AddMessage(msg,smc_r,smc_g,smc_b)
 end
end

function PriorityList:MatchBossDrop(id)
 local tmp_boss = ""
 table.foreach(boss_drops_table,
 function(k,v)
  for diff_index,diff_id_table in pairs(v) do
  for item_index,item_id in pairs(diff_id_table) do
   if item_id == id then
    tmp_boss = k..AddColor(" ("..diff_index..")",LIGHT_RED)
    break
   end
  end
  end
 end)

 if tmp_boss == "" then
  tmp_boss = L["Dropnotfound"]
 end
 return tmp_boss
end

local function UpdateItemListWidgets(itemName,itemLink,itemTexture,index)
 if itemTexture == "exists" then -- forced use of itemTexture arg :p
  opt_general.itemlist.args[index].args.descname.name = AddColor("ID #"..itemLink.." "..L["itemexists"],LIGHT_RED)
  opt_general.itemlist.args[index].args.item_label.name = AddColor("ID #"..itemLink.." "..L["itemexists"],LIGHT_RED)
  opt_general.itemlist.args[index].args.boss_label.name = AddColor("ID #"..itemLink.." "..L["itemexists"],LIGHT_RED)
 else
  if itemLink ~= nil then
   opt_general.itemlist.args[index].name = itemName
   opt_general.itemlist.args[index].args.descname.name = itemName
   opt_general.itemlist.args[index].args.item_label.name = PriorityList:ItemTooltipToString(itemLink,itemTexture)
   opt_general.itemlist.args[index].args.boss_label.name = AddColor(PriorityList:MatchBossDrop(tonumber(strsub(index,5))),GOLD)
  else
   opt_general.itemlist.args[index].name = AddColor(L["IDnotfound"],LIGHT_RED)
   opt_general.itemlist.args[index].args.descname.name = AddColor(L["IDnotfound"],LIGHT_RED)
   opt_general.itemlist.args[index].args.item_label.name = AddColor(L["IDnotfound"],LIGHT_RED)
   opt_general.itemlist.args[index].args.boss_label.name = AddColor(L["Dropnotfound"],LIGHT_RED)
  end
 end
 AceConfigRegistry:NotifyChange("PriorityList")
end

function PriorityList:PopulateItemsList()

 local function Estr_ItemProp(itemID)
 local itemName, itemLink,_,_,_,_,_,_,_, itemTexture = GetItemInfo(itemID)
 if itemID == 0 then
  itemName = AddColor(L["newitem"],LIGHT_GREEN)
 elseif itemLink == nil then
  itemName = AddColor(L["IDnotfound"],LIGHT_RED)
  waiting_ids[itemID] = {["list"] = "itemlist"}
 end
 return itemName, itemLink, itemTexture
 end --Estr_ItemProp[erties]

 local function AddItemToList(_index)

 -- Clearing useless fields (older PL lists)
 if Items_List[_index][3] ~= nil then
  Items_List[_index][3] = nil
 end
 ---------

 local itemName, itemLink, itemTexture = Estr_ItemProp(_index)
 local tooltipdesc = ""
 if itemLink then
  tooltipdesc = PriorityList:ItemTooltipToString(itemLink,itemTexture)
 elseif _index == 0 then
  tooltipdesc = AddColor(L["newitem"],LIGHT_GREEN)
  itemName = AddColor(L["newitem"],LIGHT_GREEN)
 else
  tooltipdesc = AddColor(L["IDnotfound"],LIGHT_RED)
  itemName = AddColor(L["IDnotfound"],LIGHT_RED)
 end

  opt_general.itemlist.args["item".._index] = {
   name  = itemName,
   type  = "group",
   args  = {
           descname =
          {
           order = 1,
           type  = "header",
           name  = itemName,
          },
         editID   =
          {
           order = 2,
           type  = "input",
           name  = "ID",
        validate = function(info, v) return v ~= "" end,
         confirm = pl_options["confirm_edit"],
         pattern = "^[+-]?%d+$", -- digits only
           get   = function(info)
                    return tostring(_index)
                   end,
           set   = function(info, v)
                    if tonumber(v) == 0 then
                     if Items_List[0] == nil then
                      PriorityList:AddItem(0, Items_List[_index]["needers"],true,true)
                      PriorityList:DeleteItem(_index,true)
                     end
                    else
                     if Items_List[tonumber(v)] == nil then
                      PriorityList:AddItem(tonumber(v), Items_List[_index]["needers"],true,true)
                      PriorityList:DeleteItem(_index,true)
                      _index = tonumber(v)
                      local a,b,c = Estr_ItemProp(_index)
                     elseif tonumber(v) ~= _index then
                      UpdateItemListWidgets(nil,v,"exists","item".._index)
                     end
                    end
                   end,
          },
         editchars=
          {
           order = 3,
           type  = "input",
           name  = L["needlist"],
           confirm=pl_options["confirm_edit"],
           width = "double",
           get   = function(info)
                    return Items_List[_index]["needers"]
                   end,
           set   = function(info, v)
                    Items_List[_index]["needers"] = v
                    PriorityList:PopulateItemsList()
                   end,
          },
         delitem =
          {
           order = 5,
           type  = "execute",
           name  = L["Del item"],
           desc  = L["delitemdesc"],
          confirm= pl_options["confirm_edit"],
           func  = function()
                    PriorityList:DeleteItem(_index,true)
                   end,
          },
         h1 =
          {
           order = 10,
           type  = "header",
           name  = "--- Boss ---",
          },
 boss_label =
          {
           order = 11,
           type  = "description",
           name  = AddColor(PriorityList:MatchBossDrop(_index),GOLD),
           fontSize = "large",
          },
         h2 =
          {
           order = 19,
           type  = "header",
           name  = "--- Item ---",
          },
 item_label =
          {
           order = 20,
           type  = "description",
           name  = tooltipdesc,
           fontSize = "medium",
          },
   }
  }
 end -- AddItem function

 wipe(opt_search.s_searchlist.args)
 wipe(opt_general.itemlist.args)
 PriorityList:SysMessage("Populate Items List")
 if waiting_ids then
  table.foreach(waiting_ids,function(k,v)
                             if v["list"] == "itemlist" then
                              v = {}
                             end
                            end)
 end
 if Items_List ~= nil then
  table.foreach(Items_List, AddItemToList)
 end
 collectgarbage()
end

function PriorityList:ClearList()
  wipe(Items_List)
  Items_List = {}
  DEFAULT_CHAT_FRAME:AddMessage(L["listcleared"],smc_r,smc_g,smc_b)
end

function addon:OnInitialize()
 self:RegisterChatCommand("prioritylist", "HandleSlashCommands")
 self:RegisterChatCommand("pl", "HandleSlashCommands")

 self:PopulateOptions()
 AceConfigRegistry:RegisterOptionsTable("PriorityList", addon.options)
 addon.optionsFrame = AceConfigDialog:AddToBlizOptions("PriorityList", "Priority List")
end

function PriorityList:Init()
 if Items_List == nil then
  Items_List = {
   -- default item ----------------------------------------------------
   -- Items_List[itemID] = {["needers"] = "CharA CharB CharC CharD"} --
   --------------------------------------------------------------------
  }
 end

 if Wish_List == nil then
  Wish_List = {
   -- default item ---------------------------------
   -- Wish_List["CharName"] = {ID,ID,ID,ID,ID,ID} --
   -------------------------------------------------
  }
 end

 if not pl_options then pl_options = {} end
 if pl_options["ttlinkbtn"] == nil then 
  pl_options["ttlinkbtn"] = true end -- "add item" button in any tooltip item link
 if pl_options["impsndtype"] == nil then
  pl_options["impsndtype"] = 1 end --  export list send type
 if pl_options["confirm_edit"] == nil then
  pl_options["confirm_edit"] = false end --  confirm item list changes
 if pl_options["version"] == nil or pl_options["version"] < PriorityList.version then
-- upgrading from older versions
  table.foreach(Items_List,
   function(k,v)
    if v[1] ~= nil then
     PriorityList:AddItem(v[1],v[2],false,false)
     PriorityList:DeleteItem(k,false)
    end
   end)
  pl_options["version"] = PriorityList.version
--
 end
 PriorityList:PopulateItemsList()
 PriorityList:CloseImport_Interface()
 PriorityList:Modify_Import_Widgets()
end

function addon:OnEnable()
 PriorityList:Init()
end

function addon:HandleSlashCommands(input)
 InterfaceOptionsFrame_OpenToCategory(addon.optionsFrame)
end

function PriorityList:AddItem(itemid, needlist, refresh, autoselect)
 if Items_List[itemid] == nil then
  Items_List[itemid] = {["needers"] = needlist}
  if refresh then
   PriorityList:PopulateItemsList()
   if autoselect then
    AceConfigDialog:SelectGroup("PriorityList", 'general','itemlist',"item"..itemid)
   end
  end
 end
end

function PriorityList:DeleteItem(index, refresh)
 if Items_List[index] ~= nil then
  PriorityList:SysMessage("Elimino ID: #"..index)
  Items_List[index] = nil
  if refresh then
   PriorityList:PopulateItemsList()
  end
 end
end

local PLFrame = CreateFrame("FRAME")
PLFrame:RegisterEvent("GET_ITEM_INFO_RECEIVED")
local function eventHandler(self, event, ...)
 if event == "GET_ITEM_INFO_RECEIVED" then
  PLFrame:GET_ITEM_INFO_RECEIVED()
 end
end
PLFrame:SetScript("OnEvent", eventHandler)

function PLFrame:GET_ITEM_INFO_RECEIVED()
 table.foreach(waiting_ids, function(k,v)
 local itemName, itemLink,_,_,_,_,_,_,_, itemTexture = GetItemInfo(k)
  if itemLink then
   if v["list"] == "itemlist" then
    UpdateItemListWidgets(itemName,itemLink,itemTexture,"item"..k)
--    PriorityList:PopulateItemsList()
    PriorityList:SysMessage("PL: item (MAIN LIST) received #"..k.." --> "..itemName)
   elseif v["list"] == "import" then
    if import_var.Items_List[k] ~= nil then
     if import_var.Items_List[k][3] == "mod" then
      itemName = AddColor("[MOD]-"..itemName,YELLOW)
     elseif import_var.Items_List[k][3] == "add" then
      itemName = AddColor("[ADD]-"..itemName,LIGHT_GREEN)
     end
    end
    if import_var.flagged_for_del[k] ~= nil and import_var.flagged_for_del[k] == true then
     itemName = AddColor("[DEL]-"..itemName,LIGHT_RED)
    end
     PriorityList:UpdateImportListWidgets(itemName,itemLink,itemTexture,"item"..k)
--    PriorityList:PopulateImportList()
    PriorityList:SysMessage("PL: item (IMPORT LIST) received #"..k.." --> "..itemName)
   end
   waiting_ids[k] = nil
  else
   PriorityList:SysMessage("PL: waiting for item info #"..k)
  end
 end)
end

-- loot window - item tooltip
hooksecurefunc(GameTooltip, "SetLootItem", function(tooltip, slot)
 if (LootSlotIsItem(slot)) then
  local iLink = strtrim(GetLootSlotLink(slot))

  local _, _, sLink = strfind(iLink, "^|%x+|H(.+)")
  local p, _, ItemID = strfind(sLink, "^item:(%-?%d+):?")
  if p then ItemID = tonumber(ItemID) else ItemID = nil end

  if ItemID ~= nil and ItemID > 0 then
   if Items_List[ItemID] ~= nil then
    tooltip:AddDoubleLine(L["<addon name>"]..":", GetItemInfo(ItemID))
    tooltip:AddLine(Items_List[ItemID]["needers"])
   end
  else
   tooltip:AddLine(L["IDnotfound"].." ("..L["<addon name>"]..")",1,1,1)
  end
  tooltip:Show()
 end
end)

-- hyperlink static frame - item tooltip
hooksecurefunc(ItemRefTooltip, "SetHyperlink", function (tooltip, item)
if ItemRefTooltip.PL_additem_btn then
 ItemRefTooltip.PL_additem_btn:Hide()
end
local itemname, iLink = GetItemInfo(item)
if (iLink)and(pl_options["ttlinkbtn"]) then -- it's an item
 local _, _, sLink = strfind(iLink, "^|%x+|H(.+)")
 local p, _, ItemID = strfind(sLink, "^item:(%-?%d+):?")
 if p then ItemID = tonumber(ItemID) else ItemID = nil end

 if ItemID ~= nil and ItemID > 0 then
  if Items_List[ItemID] ~= nil then
   tooltip:AddDoubleLine(L["<addon name>"]..":",L["ialreadyexists"])
   tooltip:Show()
  else
   if not ItemRefTooltip.PL_additem_btn then
    ItemRefTooltip.PL_additem_btn = CreateFrame("Button", "PLTooltipButton", tooltip, "UIPanelButtonTemplate2")
   end
   tooltip:AddLine(L["<addon name>"]..":")
   tooltip:AddDoubleLine("                    ","                    ")
   tooltip:AddLine("ID: "..ItemID)
   ItemRefTooltip.PL_additem_btn:SetPoint("BOTTOMRIGHT", tooltip, "BOTTOMRIGHT", -20, 16);
   ItemRefTooltip.PL_additem_btn:SetText(L["Add item"])
   ItemRefTooltip.PL_additem_btn:SetWidth(100)
   ItemRefTooltip.PL_additem_btn:SetScript("OnClick", function()
                                      if PriorityList.options.args.general.disabled == false then
                                       PriorityList:AddItem(ItemID,"",true,true)
                                       tooltip:Hide()
                                       InterfaceOptionsFrame_OpenToCategory(addon.optionsFrame)
                                      else
                                       DEFAULT_CHAT_FRAME:AddMessage(L["listdisabled"],smc_r,smc_g,smc_b)
                                      end
                                     end)
   ItemRefTooltip.PL_additem_btn:Show()
   tooltip:Show()
  end
 end
end
end)